# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from ...core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from ...entity_types.types.notification_type import NotificationType
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class NotificationConfigurationResponse_Invoice(UniversalBaseModel):
    notification_type: typing.Literal["invoice"] = pydantic.Field(alias="notificationType", default="invoice")
    type: NotificationType
    url: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


NotificationConfigurationResponse = NotificationConfigurationResponse_Invoice



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
import typing
from .entity_group_user_entity_request import EntityGroupUserEntityRequest
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class EntityGroupUserRequest(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.entity_group_types import (
        EntityGroupUserEntityRequest,
        EntityGroupUserRequest,
    )

    EntityGroupUserRequest(
        foreign_id="MY-DB-ID-12345",
        email="john.doe@acme.com",
        name="John Doe",
        entities=[
            EntityGroupUserEntityRequest(
                roles=["admin", "approver"],
                entity_id="ent_21661ac1-a2a8-4465-a6c0-64474ba8181d",
            ),
            EntityGroupUserEntityRequest(
                roles=["viewer"],
                entity_id="ent_9e02a20e-7749-47de-8d8a-f8ff2859fa90",
            ),
        ],
    )
    """

    foreign_id: str = pydantic.Field(alias="foreignId")
    """
    The ID used to identify this user in your system. This is a required field and needs to be unique for all users in the group.
    """

    email: typing.Optional[str] = None
    name: typing.Optional[str] = None
    entities: typing.Optional[typing.List[EntityGroupUserEntityRequest]] = pydantic.Field(default=None)
    """
    List of roles per entity. By default, the user will have no roles.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ..core.client_wrapper import SyncClientWrapper
from .notification_configuration.client import NotificationConfigurationClient
from ..core.request_options import RequestOptions
from ..organization_types.types.organization_response import OrganizationResponse
from json.decoder import JSONDecodeError
from ..core.api_error import ApiError
from ..core.pydantic_utilities import parse_obj_as
from ..commons.errors.bad_request import BadRequest
from ..commons.errors.unauthorized import Unauthorized
from ..commons.errors.forbidden import Forbidden
from ..commons.errors.not_found import NotFound
from ..commons.errors.conflict import Conflict
from ..commons.errors.internal_server_error import InternalServerError
from ..commons.errors.unimplemented import Unimplemented
from ..organization_types.types.organization_request import OrganizationRequest
import datetime as dt
from ..email_log_types.types.email_log_response import EmailLogResponse
from ..core.datetime_utils import serialize_datetime
from ..core.client_wrapper import AsyncClientWrapper
from .notification_configuration.client import AsyncNotificationConfigurationClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class OrganizationClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper
        self.notification_configuration = NotificationConfigurationClient(client_wrapper=self._client_wrapper)

    def get(
        self,
        *,
        payment_methods: typing.Optional[bool] = None,
        email_provider: typing.Optional[bool] = None,
        external_accounting_system_provider: typing.Optional[bool] = None,
        color_scheme: typing.Optional[bool] = None,
        payee_onboarding_options: typing.Optional[bool] = None,
        payor_onboarding_options: typing.Optional[bool] = None,
        metadata_schema: typing.Optional[bool] = None,
        notification_email_template: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> OrganizationResponse:
        """
        Get current organization information

        Parameters
        ----------
        payment_methods : typing.Optional[bool]
            include supported payment methods in response

        email_provider : typing.Optional[bool]
            include email provider info in response

        external_accounting_system_provider : typing.Optional[bool]
            include external accounting system provider info in response

        color_scheme : typing.Optional[bool]
            include color scheme info in response

        payee_onboarding_options : typing.Optional[bool]
            include payee onboarding options in response

        payor_onboarding_options : typing.Optional[bool]
            include payor onboarding options in response

        metadata_schema : typing.Optional[bool]
            include metadata schema in response

        notification_email_template : typing.Optional[bool]
            include notification-email-template in response

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrganizationResponse

        Examples
        --------
        from mercoa import Mercoa

        client = Mercoa(
            token="YOUR_TOKEN",
        )
        client.organization.get(
            payment_methods=True,
            email_provider=True,
            external_accounting_system_provider=True,
            color_scheme=True,
            payee_onboarding_options=True,
            payor_onboarding_options=True,
            metadata_schema=True,
            notification_email_template=True,
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "organization",
            method="GET",
            params={
                "paymentMethods": payment_methods,
                "emailProvider": email_provider,
                "externalAccountingSystemProvider": external_accounting_system_provider,
                "colorScheme": color_scheme,
                "payeeOnboardingOptions": payee_onboarding_options,
                "payorOnboardingOptions": payor_onboarding_options,
                "metadataSchema": metadata_schema,
                "notificationEmailTemplate": notification_email_template,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return typing.cast(
                OrganizationResponse,
                parse_obj_as(
                    type_=OrganizationResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self, *, request: OrganizationRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> OrganizationResponse:
        """
        Update current organization

        Parameters
        ----------
        request : OrganizationRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrganizationResponse

        Examples
        --------
        from mercoa import Mercoa
        from mercoa.organization_types import (
            BusinessOnboardingOptions,
            ColorSchemeRequest,
            EmailProviderRequest,
            EmailSenderRequest,
            ExternalAccountingSystemProviderRequest_None,
            IndividualOnboardingOptions,
            MetadataSchema,
            NotificationEmailTemplateRequest,
            OnboardingOption,
            OnboardingOptionsRequest,
            OrganizationRequest,
            PaymentMethodsRequest,
            PaymentRailRequest,
        )

        client = Mercoa(
            token="YOUR_TOKEN",
        )
        client.organization.update(
            request=OrganizationRequest(
                name="string",
                logo="string",
                website_url="string",
                support_email="string",
                payment_methods=PaymentMethodsRequest(
                    payer_payments=[
                        PaymentRailRequest(
                            type="custom",
                            active=True,
                        )
                    ],
                    backup_disbursements=[
                        PaymentRailRequest(
                            type="custom",
                            active=True,
                        )
                    ],
                    vendor_disbursements=[
                        PaymentRailRequest(
                            type="custom",
                            active=True,
                        )
                    ],
                ),
                email_provider=EmailProviderRequest(
                    sender=EmailSenderRequest(
                        provider="none",
                        from_email="string",
                        from_name="string",
                        api_key="string",
                    ),
                    inbox_domain="string",
                    alternative_inbox_domains=[],
                ),
                external_accounting_system_provider=ExternalAccountingSystemProviderRequest_None(),
                color_scheme=ColorSchemeRequest(
                    primary_color="string",
                    secondary_color="string",
                    logo_background_color="string",
                    rounded_corners=1,
                ),
                payee_onboarding_options=OnboardingOptionsRequest(
                    enable_business=True,
                    enable_individual=True,
                    payment_method=True,
                    business=BusinessOnboardingOptions(
                        type=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        doing_business_as=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        ein=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        mcc=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        formation_date=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        website=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        description=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        representatives=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        logo=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        average_transaction_size=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        average_monthly_transaction_volume=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        max_transaction_size=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        terms_of_service=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        email=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        name=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        address=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        phone=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        ten_ninety_nine=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        w_9=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                    ),
                    individual=IndividualOnboardingOptions(
                        date_of_birth=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        ssn=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        terms_of_service=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        email=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        name=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        address=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        phone=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        ten_ninety_nine=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        w_9=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                    ),
                ),
                payor_onboarding_options=OnboardingOptionsRequest(
                    enable_business=True,
                    enable_individual=True,
                    payment_method=True,
                    business=BusinessOnboardingOptions(
                        type=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        doing_business_as=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        ein=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        mcc=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        formation_date=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        website=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        description=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        representatives=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        logo=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        average_transaction_size=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        average_monthly_transaction_volume=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        max_transaction_size=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        terms_of_service=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        email=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        name=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        address=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        phone=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        ten_ninety_nine=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        w_9=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                    ),
                    individual=IndividualOnboardingOptions(
                        date_of_birth=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        ssn=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        terms_of_service=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        email=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        name=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        address=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        phone=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        ten_ninety_nine=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                        w_9=OnboardingOption(
                            show=True,
                            edit=True,
                            required=True,
                        ),
                    ),
                ),
                metadata_schema=[
                    MetadataSchema(
                        key="string",
                        display_name="string",
                        type="STRING",
                    )
                ],
                notification_email_template=NotificationEmailTemplateRequest(
                    background_style="string",
                    header="string",
                    body="string",
                    signature="string",
                    footer="string",
                    button="string",
                ),
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "organization",
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return typing.cast(
                OrganizationResponse,
                parse_obj_as(
                    type_=OrganizationResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def email_log(
        self,
        *,
        start_date: typing.Optional[dt.datetime] = None,
        end_date: typing.Optional[dt.datetime] = None,
        limit: typing.Optional[int] = None,
        starting_after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EmailLogResponse:
        """
        Get log of all emails sent to this organization. Content format subject to change.

        Parameters
        ----------
        start_date : typing.Optional[dt.datetime]

        end_date : typing.Optional[dt.datetime]

        limit : typing.Optional[int]
            Number of logs to return. Limit can range between 1 and 100, and the default is 10.

        starting_after : typing.Optional[str]
            The ID of the log to start after. If not provided, the first page of logs will be returned.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EmailLogResponse

        Examples
        --------
        import datetime

        from mercoa import Mercoa

        client = Mercoa(
            token="YOUR_TOKEN",
        )
        client.organization.email_log(
            start_date=datetime.datetime.fromisoformat(
                "2024-01-15 09:30:00+00:00",
            ),
            end_date=datetime.datetime.fromisoformat(
                "2024-01-15 09:30:00+00:00",
            ),
            limit=1,
            starting_after="string",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "organization/emailLog",
            method="GET",
            params={
                "startDate": serialize_datetime(start_date) if start_date is not None else None,
                "endDate": serialize_datetime(end_date) if end_date is not None else None,
                "limit": limit,
                "startingAfter": starting_after,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return typing.cast(
                EmailLogResponse,
                parse_obj_as(
                    type_=EmailLogResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncOrganizationClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper
        self.notification_configuration = AsyncNotificationConfigurationClient(client_wrapper=self._client_wrapper)

    async def get(
        self,
        *,
        payment_methods: typing.Optional[bool] = None,
        email_provider: typing.Optional[bool] = None,
        external_accounting_system_provider: typing.Optional[bool] = None,
        color_scheme: typing.Optional[bool] = None,
        payee_onboarding_options: typing.Optional[bool] = None,
        payor_onboarding_options: typing.Optional[bool] = None,
        metadata_schema: typing.Optional[bool] = None,
        notification_email_template: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> OrganizationResponse:
        """
        Get current organization information

        Parameters
        ----------
        payment_methods : typing.Optional[bool]
            include supported payment methods in response

        email_provider : typing.Optional[bool]
            include email provider info in response

        external_accounting_system_provider : typing.Optional[bool]
            include external accounting system provider info in response

        color_scheme : typing.Optional[bool]
            include color scheme info in response

        payee_onboarding_options : typing.Optional[bool]
            include payee onboarding options in response

        payor_onboarding_options : typing.Optional[bool]
            include payor onboarding options in response

        metadata_schema : typing.Optional[bool]
            include metadata schema in response

        notification_email_template : typing.Optional[bool]
            include notification-email-template in response

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrganizationResponse

        Examples
        --------
        import asyncio

        from mercoa import AsyncMercoa

        client = AsyncMercoa(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.organization.get(
                payment_methods=True,
                email_provider=True,
                external_accounting_system_provider=True,
                color_scheme=True,
                payee_onboarding_options=True,
                payor_onboarding_options=True,
                metadata_schema=True,
                notification_email_template=True,
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "organization",
            method="GET",
            params={
                "paymentMethods": payment_methods,
                "emailProvider": email_provider,
                "externalAccountingSystemProvider": external_accounting_system_provider,
                "colorScheme": color_scheme,
                "payeeOnboardingOptions": payee_onboarding_options,
                "payorOnboardingOptions": payor_onboarding_options,
                "metadataSchema": metadata_schema,
                "notificationEmailTemplate": notification_email_template,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return typing.cast(
                OrganizationResponse,
                parse_obj_as(
                    type_=OrganizationResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self, *, request: OrganizationRequest, request_options: typing.Optional[RequestOptions] = None
    ) -> OrganizationResponse:
        """
        Update current organization

        Parameters
        ----------
        request : OrganizationRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OrganizationResponse

        Examples
        --------
        import asyncio

        from mercoa import AsyncMercoa
        from mercoa.organization_types import (
            BusinessOnboardingOptions,
            ColorSchemeRequest,
            EmailProviderRequest,
            EmailSenderRequest,
            ExternalAccountingSystemProviderRequest_None,
            IndividualOnboardingOptions,
            MetadataSchema,
            NotificationEmailTemplateRequest,
            OnboardingOption,
            OnboardingOptionsRequest,
            OrganizationRequest,
            PaymentMethodsRequest,
            PaymentRailRequest,
        )

        client = AsyncMercoa(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.organization.update(
                request=OrganizationRequest(
                    name="string",
                    logo="string",
                    website_url="string",
                    support_email="string",
                    payment_methods=PaymentMethodsRequest(
                        payer_payments=[
                            PaymentRailRequest(
                                type="custom",
                                active=True,
                            )
                        ],
                        backup_disbursements=[
                            PaymentRailRequest(
                                type="custom",
                                active=True,
                            )
                        ],
                        vendor_disbursements=[
                            PaymentRailRequest(
                                type="custom",
                                active=True,
                            )
                        ],
                    ),
                    email_provider=EmailProviderRequest(
                        sender=EmailSenderRequest(
                            provider="none",
                            from_email="string",
                            from_name="string",
                            api_key="string",
                        ),
                        inbox_domain="string",
                        alternative_inbox_domains=[],
                    ),
                    external_accounting_system_provider=ExternalAccountingSystemProviderRequest_None(),
                    color_scheme=ColorSchemeRequest(
                        primary_color="string",
                        secondary_color="string",
                        logo_background_color="string",
                        rounded_corners=1,
                    ),
                    payee_onboarding_options=OnboardingOptionsRequest(
                        enable_business=True,
                        enable_individual=True,
                        payment_method=True,
                        business=BusinessOnboardingOptions(
                            type=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            doing_business_as=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            ein=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            mcc=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            formation_date=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            website=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            description=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            representatives=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            logo=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            average_transaction_size=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            average_monthly_transaction_volume=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            max_transaction_size=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            terms_of_service=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            email=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            name=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            address=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            phone=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            ten_ninety_nine=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            w_9=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                        ),
                        individual=IndividualOnboardingOptions(
                            date_of_birth=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            ssn=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            terms_of_service=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            email=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            name=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            address=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            phone=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            ten_ninety_nine=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            w_9=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                        ),
                    ),
                    payor_onboarding_options=OnboardingOptionsRequest(
                        enable_business=True,
                        enable_individual=True,
                        payment_method=True,
                        business=BusinessOnboardingOptions(
                            type=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            doing_business_as=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            ein=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            mcc=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            formation_date=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            website=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            description=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            representatives=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            logo=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            average_transaction_size=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            average_monthly_transaction_volume=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            max_transaction_size=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            terms_of_service=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            email=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            name=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            address=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            phone=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            ten_ninety_nine=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            w_9=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                        ),
                        individual=IndividualOnboardingOptions(
                            date_of_birth=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            ssn=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            terms_of_service=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            email=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            name=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            address=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            phone=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            ten_ninety_nine=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                            w_9=OnboardingOption(
                                show=True,
                                edit=True,
                                required=True,
                            ),
                        ),
                    ),
                    metadata_schema=[
                        MetadataSchema(
                            key="string",
                            display_name="string",
                            type="STRING",
                        )
                    ],
                    notification_email_template=NotificationEmailTemplateRequest(
                        background_style="string",
                        header="string",
                        body="string",
                        signature="string",
                        footer="string",
                        button="string",
                    ),
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "organization",
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return typing.cast(
                OrganizationResponse,
                parse_obj_as(
                    type_=OrganizationResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def email_log(
        self,
        *,
        start_date: typing.Optional[dt.datetime] = None,
        end_date: typing.Optional[dt.datetime] = None,
        limit: typing.Optional[int] = None,
        starting_after: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EmailLogResponse:
        """
        Get log of all emails sent to this organization. Content format subject to change.

        Parameters
        ----------
        start_date : typing.Optional[dt.datetime]

        end_date : typing.Optional[dt.datetime]

        limit : typing.Optional[int]
            Number of logs to return. Limit can range between 1 and 100, and the default is 10.

        starting_after : typing.Optional[str]
            The ID of the log to start after. If not provided, the first page of logs will be returned.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EmailLogResponse

        Examples
        --------
        import asyncio
        import datetime

        from mercoa import AsyncMercoa

        client = AsyncMercoa(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.organization.email_log(
                start_date=datetime.datetime.fromisoformat(
                    "2024-01-15 09:30:00+00:00",
                ),
                end_date=datetime.datetime.fromisoformat(
                    "2024-01-15 09:30:00+00:00",
                ),
                limit=1,
                starting_after="string",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "organization/emailLog",
            method="GET",
            params={
                "startDate": serialize_datetime(start_date) if start_date is not None else None,
                "endDate": serialize_datetime(end_date) if end_date is not None else None,
                "limit": limit,
                "startingAfter": starting_after,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return typing.cast(
                EmailLogResponse,
                parse_obj_as(
                    type_=EmailLogResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class MetadataRegexValidationRule(UniversalBaseModel):
    regex: str = pydantic.Field()
    """
    A regular expression that the value must match.
    """

    error_message: str = pydantic.Field(alias="errorMessage")
    """
    The error message to display if the value does not match the regular expression.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing
from ...core.client_wrapper import SyncClientWrapper
from ...entity_group_types.types.entity_group_id import EntityGroupId
from ...entity_types.types.entity_user_id import EntityUserId
from ...core.request_options import RequestOptions
from ...entity_group_types.types.find_entity_group_user_response import FindEntityGroupUserResponse
from ...core.jsonable_encoder import jsonable_encoder
from json.decoder import JSONDecodeError
from ...core.api_error import ApiError
from ...core.pydantic_utilities import parse_obj_as
from ...commons.errors.bad_request import BadRequest
from ...commons.errors.unauthorized import Unauthorized
from ...commons.errors.forbidden import Forbidden
from ...commons.errors.not_found import NotFound
from ...commons.errors.conflict import Conflict
from ...commons.errors.internal_server_error import InternalServerError
from ...commons.errors.unimplemented import Unimplemented
from ...entity_group_types.types.entity_group_user_request import EntityGroupUserRequest
from ...entity_group_types.types.entity_group_user_response import EntityGroupUserResponse
from ...entity_types.types.token_generation_options import TokenGenerationOptions
from ...core.client_wrapper import AsyncClientWrapper

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class UserClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def find(
        self,
        entity_group_id: EntityGroupId,
        *,
        foreign_id: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        role: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        name: typing.Optional[str] = None,
        email: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        starting_after: typing.Optional[EntityUserId] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FindEntityGroupUserResponse:
        """
        Search entity group users

        Parameters
        ----------
        entity_group_id : EntityGroupId
            Entity Group ID or Entity Group ForeignID

        foreign_id : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            ID used to identify user in your system

        role : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter users by role. If multiple roles are provided, users with any of the roles will be returned.

        name : typing.Optional[str]
            Filter users by name. Partial matches are supported.

        email : typing.Optional[str]
            Filter users by email. Partial matches are supported.

        limit : typing.Optional[int]
            Number of entities to return. Limit can range between 1 and 100, and the default is 10.

        starting_after : typing.Optional[EntityUserId]
            The ID of the user to start after. If not provided, the first page of entities will be returned.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FindEntityGroupUserResponse

        Examples
        --------
        from mercoa import Mercoa

        client = Mercoa(
            token="YOUR_TOKEN",
        )
        client.entity_group.user.find(
            entity_group_id="entg_8545a84e-a45f-41bf-bdf1-33b42a55812c",
            name="John",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"entityGroup/{jsonable_encoder(entity_group_id)}/users",
            method="GET",
            params={
                "foreignId": foreign_id,
                "role": role,
                "name": name,
                "email": email,
                "limit": limit,
                "startingAfter": starting_after,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return typing.cast(
                FindEntityGroupUserResponse,
                parse_obj_as(
                    type_=FindEntityGroupUserResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        entity_group_id: EntityGroupId,
        *,
        request: EntityGroupUserRequest,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EntityGroupUserResponse:
        """
        Create entity user that will be added to all entities in the group.

        Parameters
        ----------
        entity_group_id : EntityGroupId
            Entity Group ID or Entity Group ForeignID

        request : EntityGroupUserRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EntityGroupUserResponse

        Examples
        --------
        from mercoa import Mercoa
        from mercoa.entity_group_types import (
            EntityGroupUserEntityRequest,
            EntityGroupUserRequest,
        )

        client = Mercoa(
            token="YOUR_TOKEN",
        )
        client.entity_group.user.create(
            entity_group_id="entg_8545a84e-a45f-41bf-bdf1-33b42a55812c",
            request=EntityGroupUserRequest(
                foreign_id="MY-DB-ID-12345",
                email="john.doe@acme.com",
                name="John Doe",
                entities=[
                    EntityGroupUserEntityRequest(
                        roles=["admin", "approver"],
                        entity_id="ent_21661ac1-a2a8-4465-a6c0-64474ba8181d",
                    ),
                    EntityGroupUserEntityRequest(
                        roles=["viewer"],
                        entity_id="ent_9e02a20e-7749-47de-8d8a-f8ff2859fa90",
                    ),
                ],
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"entityGroup/{jsonable_encoder(entity_group_id)}/user",
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return typing.cast(
                EntityGroupUserResponse,
                parse_obj_as(
                    type_=EntityGroupUserResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(
        self,
        entity_group_id: EntityGroupId,
        foreign_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EntityGroupUserResponse:
        """
        Get entity user from a group

        Parameters
        ----------
        entity_group_id : EntityGroupId
            Entity Group ID or Entity Group ForeignID

        foreign_id : str
            ID used to identify user in your system

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EntityGroupUserResponse

        Examples
        --------
        from mercoa import Mercoa

        client = Mercoa(
            token="YOUR_TOKEN",
        )
        client.entity_group.user.get(
            entity_group_id="entg_8545a84e-a45f-41bf-bdf1-33b42a55812c",
            foreign_id="MY-DB-ID-12345",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"entityGroup/{jsonable_encoder(entity_group_id)}/user/{jsonable_encoder(foreign_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return typing.cast(
                EntityGroupUserResponse,
                parse_obj_as(
                    type_=EntityGroupUserResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        entity_group_id: EntityGroupId,
        foreign_id: str,
        *,
        request: EntityGroupUserRequest,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EntityGroupUserResponse:
        """
        Update entity user for all entities in the group.

        Parameters
        ----------
        entity_group_id : EntityGroupId
            Entity Group ID or Entity Group ForeignID

        foreign_id : str
            ID used to identify user in your system

        request : EntityGroupUserRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EntityGroupUserResponse

        Examples
        --------
        from mercoa import Mercoa
        from mercoa.entity_group_types import (
            EntityGroupUserEntityRequest,
            EntityGroupUserRequest,
        )

        client = Mercoa(
            token="YOUR_TOKEN",
        )
        client.entity_group.user.update(
            entity_group_id="entg_8545a84e-a45f-41bf-bdf1-33b42a55812c",
            foreign_id="MY-DB-ID-12345",
            request=EntityGroupUserRequest(
                foreign_id="MY-DB-ID-12345",
                email="john.doe@acme.com",
                name="John Doe",
                entities=[
                    EntityGroupUserEntityRequest(
                        roles=["admin", "approver"],
                        entity_id="ent_21661ac1-a2a8-4465-a6c0-64474ba8181d",
                    ),
                    EntityGroupUserEntityRequest(
                        roles=["viewer"],
                        entity_id="ent_9e02a20e-7749-47de-8d8a-f8ff2859fa90",
                    ),
                ],
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"entityGroup/{jsonable_encoder(entity_group_id)}/user/{jsonable_encoder(foreign_id)}",
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return typing.cast(
                EntityGroupUserResponse,
                parse_obj_as(
                    type_=EntityGroupUserResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def delete(
        self,
        entity_group_id: EntityGroupId,
        foreign_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Delete entity user from all entities in the group. This will also remove the user from all approval policies. If an approval policy will break as a result of this operation, this request will fail.

        Parameters
        ----------
        entity_group_id : EntityGroupId
            Entity Group ID or Entity Group ForeignID

        foreign_id : str
            ID used to identify user in your system

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from mercoa import Mercoa

        client = Mercoa(
            token="YOUR_TOKEN",
        )
        client.entity_group.user.delete(
            entity_group_id="string",
            foreign_id="string",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"entityGroup/{jsonable_encoder(entity_group_id)}/user/{jsonable_encoder(foreign_id)}",
            method="DELETE",
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_token(
        self,
        entity_group_id: EntityGroupId,
        foreign_id: str,
        *,
        request: TokenGenerationOptions,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> str:
        """
        Generate a JWT token for an entity group with the given options. This token can be used to authenticate to any entity in the entity group as the user in the Mercoa API and iFrame.

        Parameters
        ----------
        entity_group_id : EntityGroupId
            Entity Group ID or Entity Group ForeignID

        foreign_id : str
            ID used to identify user in your system

        request : TokenGenerationOptions

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        str

        Examples
        --------
        from mercoa import Mercoa
        from mercoa.entity_types import TokenGenerationOptions

        client = Mercoa(
            token="YOUR_TOKEN",
        )
        client.entity_group.user.get_token(
            entity_group_id="entg_a0f6ea94-0761-4a5e-a416-3c453cb7eced",
            foreign_id="MY-DB-ID-12345",
            request=TokenGenerationOptions(
                expires_in="1h",
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            f"entityGroup/{jsonable_encoder(entity_group_id)}/user/{jsonable_encoder(foreign_id)}/token",
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return typing.cast(
                str,
                parse_obj_as(
                    type_=str,  # type: ignore
                    object_=_response_json,
                ),
            )
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncUserClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def find(
        self,
        entity_group_id: EntityGroupId,
        *,
        foreign_id: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        role: typing.Optional[typing.Union[str, typing.Sequence[str]]] = None,
        name: typing.Optional[str] = None,
        email: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        starting_after: typing.Optional[EntityUserId] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FindEntityGroupUserResponse:
        """
        Search entity group users

        Parameters
        ----------
        entity_group_id : EntityGroupId
            Entity Group ID or Entity Group ForeignID

        foreign_id : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            ID used to identify user in your system

        role : typing.Optional[typing.Union[str, typing.Sequence[str]]]
            Filter users by role. If multiple roles are provided, users with any of the roles will be returned.

        name : typing.Optional[str]
            Filter users by name. Partial matches are supported.

        email : typing.Optional[str]
            Filter users by email. Partial matches are supported.

        limit : typing.Optional[int]
            Number of entities to return. Limit can range between 1 and 100, and the default is 10.

        starting_after : typing.Optional[EntityUserId]
            The ID of the user to start after. If not provided, the first page of entities will be returned.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FindEntityGroupUserResponse

        Examples
        --------
        import asyncio

        from mercoa import AsyncMercoa

        client = AsyncMercoa(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.entity_group.user.find(
                entity_group_id="entg_8545a84e-a45f-41bf-bdf1-33b42a55812c",
                name="John",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"entityGroup/{jsonable_encoder(entity_group_id)}/users",
            method="GET",
            params={
                "foreignId": foreign_id,
                "role": role,
                "name": name,
                "email": email,
                "limit": limit,
                "startingAfter": starting_after,
            },
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return typing.cast(
                FindEntityGroupUserResponse,
                parse_obj_as(
                    type_=FindEntityGroupUserResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        entity_group_id: EntityGroupId,
        *,
        request: EntityGroupUserRequest,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EntityGroupUserResponse:
        """
        Create entity user that will be added to all entities in the group.

        Parameters
        ----------
        entity_group_id : EntityGroupId
            Entity Group ID or Entity Group ForeignID

        request : EntityGroupUserRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EntityGroupUserResponse

        Examples
        --------
        import asyncio

        from mercoa import AsyncMercoa
        from mercoa.entity_group_types import (
            EntityGroupUserEntityRequest,
            EntityGroupUserRequest,
        )

        client = AsyncMercoa(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.entity_group.user.create(
                entity_group_id="entg_8545a84e-a45f-41bf-bdf1-33b42a55812c",
                request=EntityGroupUserRequest(
                    foreign_id="MY-DB-ID-12345",
                    email="john.doe@acme.com",
                    name="John Doe",
                    entities=[
                        EntityGroupUserEntityRequest(
                            roles=["admin", "approver"],
                            entity_id="ent_21661ac1-a2a8-4465-a6c0-64474ba8181d",
                        ),
                        EntityGroupUserEntityRequest(
                            roles=["viewer"],
                            entity_id="ent_9e02a20e-7749-47de-8d8a-f8ff2859fa90",
                        ),
                    ],
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"entityGroup/{jsonable_encoder(entity_group_id)}/user",
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return typing.cast(
                EntityGroupUserResponse,
                parse_obj_as(
                    type_=EntityGroupUserResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(
        self,
        entity_group_id: EntityGroupId,
        foreign_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EntityGroupUserResponse:
        """
        Get entity user from a group

        Parameters
        ----------
        entity_group_id : EntityGroupId
            Entity Group ID or Entity Group ForeignID

        foreign_id : str
            ID used to identify user in your system

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EntityGroupUserResponse

        Examples
        --------
        import asyncio

        from mercoa import AsyncMercoa

        client = AsyncMercoa(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.entity_group.user.get(
                entity_group_id="entg_8545a84e-a45f-41bf-bdf1-33b42a55812c",
                foreign_id="MY-DB-ID-12345",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"entityGroup/{jsonable_encoder(entity_group_id)}/user/{jsonable_encoder(foreign_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return typing.cast(
                EntityGroupUserResponse,
                parse_obj_as(
                    type_=EntityGroupUserResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        entity_group_id: EntityGroupId,
        foreign_id: str,
        *,
        request: EntityGroupUserRequest,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> EntityGroupUserResponse:
        """
        Update entity user for all entities in the group.

        Parameters
        ----------
        entity_group_id : EntityGroupId
            Entity Group ID or Entity Group ForeignID

        foreign_id : str
            ID used to identify user in your system

        request : EntityGroupUserRequest

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        EntityGroupUserResponse

        Examples
        --------
        import asyncio

        from mercoa import AsyncMercoa
        from mercoa.entity_group_types import (
            EntityGroupUserEntityRequest,
            EntityGroupUserRequest,
        )

        client = AsyncMercoa(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.entity_group.user.update(
                entity_group_id="entg_8545a84e-a45f-41bf-bdf1-33b42a55812c",
                foreign_id="MY-DB-ID-12345",
                request=EntityGroupUserRequest(
                    foreign_id="MY-DB-ID-12345",
                    email="john.doe@acme.com",
                    name="John Doe",
                    entities=[
                        EntityGroupUserEntityRequest(
                            roles=["admin", "approver"],
                            entity_id="ent_21661ac1-a2a8-4465-a6c0-64474ba8181d",
                        ),
                        EntityGroupUserEntityRequest(
                            roles=["viewer"],
                            entity_id="ent_9e02a20e-7749-47de-8d8a-f8ff2859fa90",
                        ),
                    ],
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"entityGroup/{jsonable_encoder(entity_group_id)}/user/{jsonable_encoder(foreign_id)}",
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return typing.cast(
                EntityGroupUserResponse,
                parse_obj_as(
                    type_=EntityGroupUserResponse,  # type: ignore
                    object_=_response_json,
                ),
            )
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def delete(
        self,
        entity_group_id: EntityGroupId,
        foreign_id: str,
        *,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Delete entity user from all entities in the group. This will also remove the user from all approval policies. If an approval policy will break as a result of this operation, this request will fail.

        Parameters
        ----------
        entity_group_id : EntityGroupId
            Entity Group ID or Entity Group ForeignID

        foreign_id : str
            ID used to identify user in your system

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from mercoa import AsyncMercoa

        client = AsyncMercoa(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.entity_group.user.delete(
                entity_group_id="string",
                foreign_id="string",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"entityGroup/{jsonable_encoder(entity_group_id)}/user/{jsonable_encoder(foreign_id)}",
            method="DELETE",
            request_options=request_options,
        )
        if 200 <= _response.status_code < 300:
            return
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_token(
        self,
        entity_group_id: EntityGroupId,
        foreign_id: str,
        *,
        request: TokenGenerationOptions,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> str:
        """
        Generate a JWT token for an entity group with the given options. This token can be used to authenticate to any entity in the entity group as the user in the Mercoa API and iFrame.

        Parameters
        ----------
        entity_group_id : EntityGroupId
            Entity Group ID or Entity Group ForeignID

        foreign_id : str
            ID used to identify user in your system

        request : TokenGenerationOptions

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        str

        Examples
        --------
        import asyncio

        from mercoa import AsyncMercoa
        from mercoa.entity_types import TokenGenerationOptions

        client = AsyncMercoa(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.entity_group.user.get_token(
                entity_group_id="entg_a0f6ea94-0761-4a5e-a416-3c453cb7eced",
                foreign_id="MY-DB-ID-12345",
                request=TokenGenerationOptions(
                    expires_in="1h",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"entityGroup/{jsonable_encoder(entity_group_id)}/user/{jsonable_encoder(foreign_id)}/token",
            method="POST",
            json=request,
            request_options=request_options,
            omit=OMIT,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return typing.cast(
                str,
                parse_obj_as(
                    type_=str,  # type: ignore
                    object_=_response_json,
                ),
            )
        if "errorName" in _response_json:
            if _response_json["errorName"] == "BadRequest":
                raise BadRequest(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unauthorized":
                raise Unauthorized(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Forbidden":
                raise Forbidden(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "NotFound":
                raise NotFound(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Conflict":
                raise Conflict(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "InternalServerError":
                raise InternalServerError(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
            if _response_json["errorName"] == "Unimplemented":
                raise Unimplemented(
                    typing.cast(
                        str,
                        parse_obj_as(
                            type_=str,  # type: ignore
                            object_=_response_json["content"],
                        ),
                    )
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from .vendor_network import VendorNetwork
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class TokenGenerationVendorOptions(UniversalBaseModel):
    disable_creation: typing.Optional[bool] = pydantic.Field(alias="disableCreation", default=None)
    """
    If true, the user will not be able to create new vendors.
    """

    network: VendorNetwork

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from .token_generation_invoice_options import TokenGenerationInvoiceOptions
from .token_generation_pages_options import TokenGenerationPagesOptions
from .token_generation_style_options import TokenGenerationStyleOptions
from .token_generation_vendor_options import TokenGenerationVendorOptions
from .token_generation_entity_options import TokenGenerationEntityOptions
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class TokenGenerationOptions(UniversalBaseModel):
    expires_in: typing.Optional[str] = pydantic.Field(alias="expiresIn", default=None)
    """
    Expressed in seconds or a string describing a time span. The default is 1h.
    """

    invoice: typing.Optional[TokenGenerationInvoiceOptions] = None
    pages: typing.Optional[TokenGenerationPagesOptions] = None
    style: typing.Optional[TokenGenerationStyleOptions] = None
    vendors: typing.Optional[TokenGenerationVendorOptions] = None
    entity: typing.Optional[TokenGenerationEntityOptions] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
import typing
from ...payment_method_types.types.currency_code import CurrencyCode
from ...payment_method_types.types.payment_method_id import PaymentMethodId
from ...invoice_types.types.payment_destination_options import PaymentDestinationOptions
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class CalculateFeesRequest(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.calculate import CalculateFeesRequest

    CalculateFeesRequest(
        amount=100.0,
        payment_source_id="pm_4794d597-70dc-4fec-b6ec-c5988e759769",
        payment_destination_id="pm_4794d597-70dc-4fec-b6ec-c5988e759769",
    )
    """

    amount: float = pydantic.Field()
    """
    Total amount in major units. If the entered amount has more decimal places than the currency supports, trailing decimals will be truncated.
    """

    currency: typing.Optional[CurrencyCode] = pydantic.Field(default=None)
    """
    Currency code for the amount. Defaults to USD.
    """

    payment_source_id: PaymentMethodId = pydantic.Field(alias="paymentSourceId")
    """
    ID of payment source.
    """

    payment_destination_id: PaymentMethodId = pydantic.Field(alias="paymentDestinationId")
    """
    ID of payment destination.
    """

    payment_destination_options: typing.Optional[PaymentDestinationOptions] = pydantic.Field(
        alias="paymentDestinationOptions", default=None
    )
    """
    Options for the payment destination. Depending on the payment destination, this may include things such as check delivery method.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from ...core.pydantic_utilities import UniversalBaseModel
import typing
from ...payment_method_types.types.currency_code import CurrencyCode
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import pydantic
from .entity_id import EntityId


class Trigger_Amount(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.entity_types import Trigger_Amount

    Trigger_Amount(
        amount=100.0,
        currency="USD",
    )
    """

    type: typing.Literal["amount"] = "amount"
    amount: float
    currency: CurrencyCode

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class Trigger_Vendor(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.entity_types import Trigger_Amount

    Trigger_Amount(
        amount=100.0,
        currency="USD",
    )
    """

    type: typing.Literal["vendor"] = "vendor"
    vendor_ids: typing.List[EntityId] = pydantic.Field(alias="vendorIds")

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class Trigger_Metadata(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.entity_types import Trigger_Amount

    Trigger_Amount(
        amount=100.0,
        currency="USD",
    )
    """

    type: typing.Literal["metadata"] = "metadata"
    key: str
    value: str

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


"""
from mercoa.entity_types import Trigger_Amount

Trigger_Amount(
    amount=100.0,
    currency="USD",
)
"""
Trigger = typing.Union[Trigger_Amount, Trigger_Vendor, Trigger_Metadata]



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class PaymentMethodBaseRequest(UniversalBaseModel):
    default_source: typing.Optional[bool] = pydantic.Field(alias="defaultSource", default=None)
    """
    If true, this payment method will be set as the default source. Only one payment method can be set as the default source. If another payment method is already set as the default source, it will be unset.
    """

    default_destination: typing.Optional[bool] = pydantic.Field(alias="defaultDestination", default=None)
    """
    If true, this payment method will be set as the default destination. Only one payment method can be set as the default destination. If another payment method is already set as the default destination, it will be unset.
    """

    external_accounting_system_id: typing.Optional[str] = pydantic.Field(
        alias="externalAccountingSystemId", default=None
    )
    """
    ID for this payment method in the external accounting system (e.g Rutter or Codat)
    """

    frozen: typing.Optional[bool] = pydantic.Field(default=None)
    """
    If true, this payment method will be frozen. Frozen payment methods cannot be used for payments, but will still be returned in API responses.
    """

    metadata: typing.Optional[typing.Dict[str, str]] = pydantic.Field(default=None)
    """
    Metadata associated with this payment method.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

InvoiceDateFilter = typing.Union[
    typing.Literal[
        "INVOICE_DATE", "DEDUCTION_DATE", "NEXT_PAYMENT_DATE", "DUE_DATE", "SETTLEMENT_DATE", "CREATED_AT", "UPDATED_AT"
    ],
    typing.Any,
]



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
from .metadata_customization_request import MetadataCustomizationRequest
from .payment_method_customization_request import PaymentMethodCustomizationRequest
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class EntityCustomizationRequest(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.entity_types import (
        EntityCustomizationRequest,
        MetadataCustomizationRequest,
        PaymentMethodCustomizationRequest,
    )

    EntityCustomizationRequest(
        metadata=[
            MetadataCustomizationRequest(
                key="my_custom_field",
                disabled=True,
            ),
            MetadataCustomizationRequest(
                key="my_other_field",
                disabled=False,
            ),
        ],
        payment_source=[
            PaymentMethodCustomizationRequest(
                type="bankAccount",
                disabled=True,
            ),
            PaymentMethodCustomizationRequest(
                type="custom",
                schema_id="cpms_7df2974a-4069-454c-912f-7e58ebe030fb",
                disabled=True,
            ),
        ],
        backup_disbursement=[
            PaymentMethodCustomizationRequest(
                type="check",
                disabled=True,
            )
        ],
        payment_destination=[
            PaymentMethodCustomizationRequest(
                type="bankAccount",
                disabled=True,
            ),
            PaymentMethodCustomizationRequest(
                type="check",
                disabled=True,
            ),
        ],
    )
    """

    metadata: typing.List[MetadataCustomizationRequest]
    payment_source: typing.List[PaymentMethodCustomizationRequest] = pydantic.Field(alias="paymentSource")
    backup_disbursement: typing.List[PaymentMethodCustomizationRequest] = pydantic.Field(alias="backupDisbursement")
    payment_destination: typing.List[PaymentMethodCustomizationRequest] = pydantic.Field(alias="paymentDestination")

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
from .representative_id import RepresentativeId
from ...commons.types.full_name import FullName
import typing
from ...commons.types.phone_number import PhoneNumber
from ...commons.types.address import Address
import pydantic
from .responsibilities import Responsibilities
import datetime as dt
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class RepresentativeResponse(UniversalBaseModel):
    """
    Examples
    --------
    import datetime

    from mercoa.commons import Address, FullName, PhoneNumber
    from mercoa.entity_types import RepresentativeResponse, Responsibilities

    RepresentativeResponse(
        id="rep_7df2974a-4069-454c-912f-7e58ebe030fb",
        name=FullName(
            first_name="John",
            middle_name="Quincy",
            last_name="Adams",
            suffix="Jr.",
        ),
        phone=PhoneNumber(
            country_code="1",
            number="4155551234",
        ),
        email="john.doe@acme.com",
        address=Address(
            address_line_1="123 Main St",
            address_line_2="Unit 1",
            city="San Francisco",
            state_or_province="CA",
            postal_code="94105",
            country="US",
        ),
        birth_date_provided=True,
        government_id_provided=True,
        responsibilities=Responsibilities(
            is_owner=True,
            ownership_percentage=40,
            is_controller=True,
        ),
        created_on=datetime.datetime.fromisoformat(
            "2024-01-01 00:00:00+00:00",
        ),
        updated_on=datetime.datetime.fromisoformat(
            "2024-01-01 00:00:00+00:00",
        ),
    )
    """

    id: RepresentativeId
    name: FullName
    phone: typing.Optional[PhoneNumber] = None
    email: typing.Optional[str] = None
    address: Address
    birth_date_provided: bool = pydantic.Field(alias="birthDateProvided")
    government_id_provided: bool = pydantic.Field(alias="governmentIDProvided")
    responsibilities: Responsibilities
    created_on: dt.datetime = pydantic.Field(alias="createdOn")
    updated_on: dt.datetime = pydantic.Field(alias="updatedOn")
    disabled_on: typing.Optional[dt.datetime] = pydantic.Field(alias="disabledOn", default=None)

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from ...core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class ExternalAccountingSystemProviderRequest_None(UniversalBaseModel):
    type: typing.Literal["none"] = "none"
    api_key: str = pydantic.Field(alias="apiKey")

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ExternalAccountingSystemProviderRequest_Codat(UniversalBaseModel):
    type: typing.Literal["codat"] = "codat"
    api_key: str = pydantic.Field(alias="apiKey")

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class ExternalAccountingSystemProviderRequest_Rutter(UniversalBaseModel):
    type: typing.Literal["rutter"] = "rutter"
    client_id: str = pydantic.Field(alias="clientId")
    client_secret: str = pydantic.Field(alias="clientSecret")

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


ExternalAccountingSystemProviderRequest = typing.Union[
    ExternalAccountingSystemProviderRequest_None,
    ExternalAccountingSystemProviderRequest_Codat,
    ExternalAccountingSystemProviderRequest_Rutter,
]



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
from .custom_payment_method_schema_id import CustomPaymentMethodSchemaId
import pydantic
import typing
from .currency_code import CurrencyCode
from .custom_payment_method_schema_field import CustomPaymentMethodSchemaField
from .custom_payment_method_schema_fee import CustomPaymentMethodSchemaFee
import datetime as dt
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class CustomPaymentMethodSchemaResponse(UniversalBaseModel):
    """
    Examples
    --------
    import datetime

    from mercoa.payment_method_types import (
        CustomPaymentMethodSchemaField,
        CustomPaymentMethodSchemaResponse,
    )

    CustomPaymentMethodSchemaResponse(
        id="cpms_4794d597-70dc-4fec-b6ec-c5988e759769",
        name="Wire",
        is_source=False,
        is_destination=True,
        supported_currencies=["USD", "EUR"],
        fields=[
            CustomPaymentMethodSchemaField(
                name="bankName",
                display_name="Bank Name",
                type="text",
                optional=False,
            ),
            CustomPaymentMethodSchemaField(
                name="recipientName",
                display_name="Recipient Name",
                type="text",
                optional=False,
            ),
            CustomPaymentMethodSchemaField(
                name="accountNumber",
                display_name="Account Number",
                type="usBankAccountNumber",
                optional=False,
                use_as_account_number=True,
            ),
            CustomPaymentMethodSchemaField(
                name="routingNumber",
                display_name="Routing Number",
                type="usBankRoutingNumber",
                optional=False,
            ),
        ],
        created_at=datetime.datetime.fromisoformat(
            "2021-01-01 00:00:00+00:00",
        ),
        updated_at=datetime.datetime.fromisoformat(
            "2021-01-01 00:00:00+00:00",
        ),
        estimated_processing_time=0,
        max_amount=100000.0,
        min_amount=1.0,
    )
    """

    id: CustomPaymentMethodSchemaId
    name: str
    is_source: bool = pydantic.Field(alias="isSource")
    """
    This payment method can be used as a payment source for an invoice
    """

    is_destination: bool = pydantic.Field(alias="isDestination")
    """
    This payment method can be used as a payment destination for an invoice
    """

    supported_currencies: typing.List[CurrencyCode] = pydantic.Field(alias="supportedCurrencies")
    """
    List of currencies that this payment method supports.
    """

    fields: typing.List[CustomPaymentMethodSchemaField]
    estimated_processing_time: int = pydantic.Field(alias="estimatedProcessingTime")
    """
    Estimated time in days for this payment method to process a payments. 0 is an same-day payment methods, -1 is unknown processing time.
    """

    max_amount: typing.Optional[float] = pydantic.Field(alias="maxAmount", default=None)
    """
    The maximum amount that can be transferred from this payment method in a single transaction.
    """

    min_amount: typing.Optional[float] = pydantic.Field(alias="minAmount", default=None)
    """
    The minimum amount that can be transferred from this payment method in a single transaction. Default is 1.
    """

    fees: typing.Optional[CustomPaymentMethodSchemaFee] = None
    created_at: dt.datetime = pydantic.Field(alias="createdAt")
    updated_at: dt.datetime = pydantic.Field(alias="updatedAt")

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2
import typing


class MetadataCustomizationRequest(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.entity_types import MetadataCustomizationRequest

    MetadataCustomizationRequest(
        key="my_custom_field",
        disabled=True,
    )
    """

    key: str = pydantic.Field()
    """
    The key of the metadata field. This must be defined at the organization level, otherwise an error will be returned.
    """

    disabled: bool = pydantic.Field()
    """
    If true, this field will not be available to the entity.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations
from ...core.pydantic_utilities import UniversalBaseModel
import typing
import pydantic
from .payment_schedule_end_condition import PaymentScheduleEndCondition
from ...core.pydantic_utilities import IS_PYDANTIC_V2
from .day_of_week import DayOfWeek
from .payment_month_repeat_type import PaymentMonthRepeatType


class PaymentSchedule_OneTime(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.invoice_types import PaymentSchedule_OneTime

    PaymentSchedule_OneTime()
    """

    type: typing.Literal["oneTime"] = "oneTime"
    repeat_every: typing.Optional[int] = pydantic.Field(alias="repeatEvery", default=None)
    ends: typing.Optional[PaymentScheduleEndCondition] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class PaymentSchedule_Daily(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.invoice_types import PaymentSchedule_OneTime

    PaymentSchedule_OneTime()
    """

    type: typing.Literal["daily"] = "daily"
    repeat_every: typing.Optional[int] = pydantic.Field(alias="repeatEvery", default=None)
    ends: typing.Optional[PaymentScheduleEndCondition] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class PaymentSchedule_Weekly(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.invoice_types import PaymentSchedule_OneTime

    PaymentSchedule_OneTime()
    """

    type: typing.Literal["weekly"] = "weekly"
    repeat_on: typing.List[DayOfWeek] = pydantic.Field(alias="repeatOn")
    repeat_every: typing.Optional[int] = pydantic.Field(alias="repeatEvery", default=None)
    ends: typing.Optional[PaymentScheduleEndCondition] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class PaymentSchedule_Monthly(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.invoice_types import PaymentSchedule_OneTime

    PaymentSchedule_OneTime()
    """

    type: typing.Literal["monthly"] = "monthly"
    day_offset: int = pydantic.Field(alias="dayOffset")
    offset_type: typing.Optional[PaymentMonthRepeatType] = pydantic.Field(alias="offsetType", default=None)
    repeat_every: typing.Optional[int] = pydantic.Field(alias="repeatEvery", default=None)
    ends: typing.Optional[PaymentScheduleEndCondition] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


class PaymentSchedule_Yearly(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.invoice_types import PaymentSchedule_OneTime

    PaymentSchedule_OneTime()
    """

    type: typing.Literal["yearly"] = "yearly"
    repeat_on_day: int = pydantic.Field(alias="repeatOnDay")
    repeat_on_month: int = pydantic.Field(alias="repeatOnMonth")
    repeat_every: typing.Optional[int] = pydantic.Field(alias="repeatEvery", default=None)
    ends: typing.Optional[PaymentScheduleEndCondition] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow


"""
from mercoa.invoice_types import PaymentSchedule_OneTime

PaymentSchedule_OneTime()
"""
PaymentSchedule = typing.Union[
    PaymentSchedule_OneTime,
    PaymentSchedule_Daily,
    PaymentSchedule_Weekly,
    PaymentSchedule_Monthly,
    PaymentSchedule_Yearly,
]



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
from ...commons.types.full_name import FullName
from ...commons.types.phone_number import PhoneNumber
from ...commons.types.address import Address
from ...commons.types.birth_date import BirthDate
import pydantic
from ...commons.types.individual_government_id import IndividualGovernmentId
from .responsibilities import Responsibilities
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class RepresentativeUpdateRequest(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.commons import (
        Address,
        BirthDate,
        FullName,
        IndividualGovernmentId,
        PhoneNumber,
    )
    from mercoa.entity_types import RepresentativeUpdateRequest, Responsibilities

    RepresentativeUpdateRequest(
        name=FullName(
            first_name="John",
            middle_name="Quincy",
            last_name="Adams",
            suffix="Jr.",
        ),
        phone=PhoneNumber(
            country_code="1",
            number="4155551234",
        ),
        email="john.doe@acme.com",
        address=Address(
            address_line_1="123 Main St",
            address_line_2="Unit 1",
            city="San Francisco",
            state_or_province="CA",
            postal_code="94105",
            country="US",
        ),
        birth_date=BirthDate(
            day="1",
            month="1",
            year="1980",
        ),
        government_id=IndividualGovernmentId(
            ssn="123-45-6789",
        ),
        responsibilities=Responsibilities(
            is_owner=True,
            ownership_percentage=40,
            is_controller=True,
        ),
    )
    """

    name: typing.Optional[FullName] = None
    phone: typing.Optional[PhoneNumber] = None
    email: typing.Optional[str] = None
    address: typing.Optional[Address] = None
    birth_date: typing.Optional[BirthDate] = pydantic.Field(alias="birthDate", default=None)
    government_id: typing.Optional[IndividualGovernmentId] = pydantic.Field(alias="governmentID", default=None)
    responsibilities: typing.Optional[Responsibilities] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
from .email_template_type import EmailTemplateType
import pydantic
import typing
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class EmailTemplateRequest(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.entity_types import EmailTemplateRequest

    EmailTemplateRequest(
        template_type="PAYMENT",
        name="Generic Payment Email",
        subject="Action Required - Your payment is due",
        content="<h1>Your invoice has been sent.</h1>",
        is_default=True,
    )
    """

    template_type: EmailTemplateType = pydantic.Field(alias="templateType")
    name: str = pydantic.Field()
    """
    The name of the email template.
    """

    subject: str = pydantic.Field()
    """
    The subject of the email template.
    """

    content: str = pydantic.Field()
    """
    The HTML content of the email template.
    """

    is_default: typing.Optional[bool] = pydantic.Field(alias="isDefault", default=None)
    """
    If true, this email template will be used as the default template for new invoices.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ...core.pydantic_utilities import UniversalBaseModel
import typing
from .check_delivery_method import CheckDeliveryMethod
import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2


class CheckPaymentDestinationOptions(UniversalBaseModel):
    """
    Examples
    --------
    from mercoa.invoice_types import CheckPaymentDestinationOptions

    CheckPaymentDestinationOptions(
        delivery="MAIL",
    )
    """

    delivery: typing.Optional[CheckDeliveryMethod] = pydantic.Field(default=None)
    """
    Delivery method for check disbursements. Defaults to MAIL.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
